<?php
require 'Zend/Loader.php';

/**
 * The Bootstrap
 *
 * @author TAAT
 * @link http://taat.pl
 * @version 1.0
 */
class Bootstrap
{
    // PROPERTIES
    /**
     * Front controller instance
     * @access public static
     * @var object Zend_Controller_Front
     */
    public static $frontController = null;

    /**
     * root directory
     * @access public static
     * @var string
     */
    public static $root = '';

    /**
     * Run app
     * @access public static
     * @return null
     */
    // PUBLIC METHODS
    public static function run ()
    {
		Zend_Loader::registerAutoload();
        self::setup();
        $response = self::$frontController->dispatch();
        self::sendResponse($response);
    }

    /**
     * Prepare Front Controller and View
     * @access public static
     * @return null
     */
    public static function setup()
    {
		self::setupConfig();
		self::setupEnvironment();
        self::setupFrontController();
        self::setupView();
		self::setupDatabase();
    }

    /**
     * setup environment
     * @access public static
     * @param
     * @return null
     */
    public static function setupEnvironment ()
    {
        self::$root = dirname(dirname(__FILE__));
        set_include_path(
                         get_include_path() .
                         PATH_SEPARATOR . self::$root . '/library/'
                        );
		// doctrine paths
		$doctrine_models = Zend_Registry::get('config')->doctrine->models_path;
		set_include_path(
                         get_include_path()
						 . PATH_SEPARATOR .  self::$root . '/' . $doctrine_models
						 . PATH_SEPARATOR .  self::$root . '/' . $doctrine_models . '/generated'
						 );
    }

	/**
	* Read config file
	* @access public
	* @param string $configfile Relative path to config file
	* @param bool $debug null = detect from ENV
	* @return null
	*/
	public static function setupConfig($configfile = '/application/config/config.ini') {
		$configSection = strtolower(
									in_array(
											 getenv('DEVELOPMENT'), array('on', 'true', 1))
									) ? 'development' : 'live';
		$config = new Zend_Config_Ini(dirname(dirname(__FILE__)) . $configfile, $configSection);
		Zend_Registry::set('config', $config);
		date_default_timezone_set($config->timezone);
		if ($config->debug) {
			ini_set('display_errors', true);
			error_reporting(E_ALL | E_STRICT);
		} else {
			ini_set('display_errors', false);
			error_reporting(0);
		}
	}

    /**
     * Prepare front controller
     * @access public static
     * @param
     * @return null
     */
    public static function setupFrontController ()
    {
        self::$frontController = Zend_Controller_Front::getInstance();
        self::$frontController->throwExceptions(true);
        self::$frontController->setParam('noErrorHandler', false);
        self::$frontController->returnResponse(true);
        self::$frontController->addControllerDirectory(self::$root .  '/application/modules/default/controllers');
    }

    /**
     * Setup View
     * @access public static
     * @return null
     */
    public static function setupView ()
    {
        self::$frontController->registerPlugin(new My_Controller_Plugin_ViewSetup());
    }

	/**
	* Setup Database
	* @access public static
	* @return null
	*/
    public static function setupDatabase ()
    {
        My_Controller_Plugin_Doctrine::setupDoctrine();
		self::setupFirePHP(Zend_Registry::get('config')->debug);
        My_Controller_Plugin_Doctrine::setupSession();
    }

    /**
     * Send response to the browser
     * @access public static
     * @param  Zend_Controller_Response_Http $response
     * @return null
     */
    public static function sendResponse (Zend_Controller_Response_Http $response)
    {
        $response->setHeader('Content-Type', 'text/html; charset=UTF-8');
        // $response->setHeader( 'Accept-encoding', 'gzip,deflate');
        $response->sendResponse();

		// Display Doctrine Profiling information in Firebug
		Zend_Wildfire_Channel_HttpHeaders::getInstance()->flush();
    }

	/**
	 * setup FirePHP
	 * @access public static
	 * @param bool $enable Enable or disable logging
	 * @return null
	 */
	public static function setupFirePHP($enabled = true) {
		$logger = new Zend_Log();
		$writer = new Zend_Log_Writer_Firebug();
		$logger->addWriter($writer);
		// add FirePHP logger to the registry
		Zend_Registry::set('logger', $logger);

		if ($enabled) {
			$writer->setEnabled($enabled);
			// setup Doctrine profiling
			$profiler = new Imind_Profiler_Doctrine_Firebug('All DB Queries: ');
			Doctrine_Manager::connection()->setListener($profiler);
			ob_start();
		}
	}
}

/**
 * FirePHP debugging shortcut
 */
function fb($message, $label = null)
{
	$logger = Zend_Registry::get('logger');
	if ($label != null) {
		$logger->log($label . ':', Zend_Log::INFO);
	}
	$logger->log($message, Zend_Log::DEBUG);
}